import json
import requests
import pandas as pd
from pydantic import BaseModel

from pygeoweaver.constants import *
from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
    check_ipython,
)


class ProcessData(BaseModel):
    type: str = "process"
    lang: str
    description: str
    name: str
    code: str
    owner: str = "111111"
    confidential: bool = False


class WorkflowData(BaseModel):
    type: str = "workflow"
    confidential: bool = False
    description: str
    edges: str
    name: str
    nodes: str
    owner: str = "111111"


def create_process(lang, description, name, code, owner="111111", confidential=False):
    """
        Function to create a process with given data if valid.
    :param lang: The programming language of the process
    :type lang: str
    :param description: The description of the process
    :type description: str
    :param name: The name of the process
    :type name: str
    :param code: The code of the process
    :type code: str
    :param owner: The owner of the process, defaults to "111111"
    :type owner: str, optional
    :param confidential: The confidentiality status of the process, defaults to False
    :type confidential: bool, optional
    :return: Returns the id of the created process
    :rtype: dict
    """
    download_geoweaver_jar()
    process = ProcessData(
        type="process",
        lang=lang,
        description=description,
        name=name,
        code=code,
        owner=owner,
        confidential=confidential,
    )
    data_json = process.json()
    r = requests.post(
        f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/add/process",
        data=data_json,
        headers=COMMON_API_HEADER,
    )
    if check_ipython() and r.ok:
        df = pd.DataFrame(json.loads(data_json).items(), columns=["Key", "Value"])
        return df
    else:
        return r.json()


def create_process_from_file(
    lang, description, name, file_path, owner="111111", confidential=False
):
    """
    Function to create a process with code from a file.
    :param lang: The programming language of the process.
    :type lang: str
    :param description: The description of the process.
    :type description: str
    :param name: The name of the process.
    :type name: str
    :param file_path: The path to the file containing the code.
    :type file_path: str
    :param owner: The owner of the process, defaults to "111111".
    :type owner: str, optional
    :param confidential: The confidentiality status of the process, defaults to False.
    :type confidential: bool, optional
    :return: Returns the id of the created process.
    :rtype: dict
    """
    with open(file_path, "r") as file:
        code = file.read()
    return create_process(
        lang, description, name, code, owner=owner, confidential=confidential
    )


def create_workflow(
    description, edges, name, nodes, owner="111111", confidential=False
):
    """
        Function to create a workflow with given data if valid
    :param confidential: The confidentiality status of the workflow, defaults to False
    :type confidential: bool, optional
    :param description: The description of the workflow
    :type description: str
    :param edges: The edges of the workflow
    :type edges: str
    :param name: The name of the workflow
    :type name: str
    :param nodes: The nodes of the workflow
    :type nodes: str
    :param owner: The owner of the workflow, defaults to "111111"
    :type owner: str, optional
    :return: Returns the id of the created workflow
    :rtype: dict
    """
    download_geoweaver_jar()
    workflow = WorkflowData(
        type="workflow",
        confidential=confidential,
        description=description,
        edges=edges,
        name=name,
        nodes=nodes,
        owner=owner,
    )
    data_json = workflow.json()
    r = requests.post(
        f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/add/workflow",
        data=data_json,
        headers=COMMON_API_HEADER,
    )
    if check_ipython():
        return pd.DataFrame(json.loads(data_json).items(), columns=["Key", "Value"])
    else:
        return r.json()



"""
The main function of pygeoweaver
To run in CLI mode. 
"""
from pygeoweaver import (
    detail_host,
    detail_process,
    detail_workflow,
    export_workflow,
    show_history,
    import_workflow,
    list_hosts,
    list_processes,
    list_workflows,
    start,
    stop,
    reset_password,
    run_process,
    run_workflow,
    helpwith,
)
from pygeoweaver.server import show


def main():
    # start geoweaver
    # start()
    # stop geoweaver
    # stop()
    # list resources
    # list_hosts()
    # list_processes()
    # list_workflows()
    # show history
    # show_history("ll3u3W78eOEfklxhBJ")
    # detail host
    # detail_host("100001")
    # detail process
    # detail_process("7pxu8c")
    # detail_workflow("5jnhcrq33znbu2mue9v2")
    # import workflow
    # import_workflow("/Users/joe/Downloads/gr3ykr8dynu12vrwq11oy.zip")
    # export workflow
    # export_workflow("gr3ykr8dynu12vrwq11oy", "4", "/Users/joe/Downloads/test_pygeoweaver_export.zip")
    # run process
    # run_process(process_id="7zwnvx", host_id="100001", )
    # run workflow by id
    # run_worklfow(workflow_id="9sszomwhiiusakodb1ft", host_list="100001", password_list="xxx",
    #              environment_list="",)
    # run_workflow(workflow_id="9sszomwhiiusakodb1ft", host_list="100001", )
    # run workflow by zip path

    # run workflow by folder path

    # reset localhost password for Geoweaver
    # reset_password()
    # show()
    # helpwith()
    # check_java()
    pass


if __name__ == "__main__":
    main()



"""
The main function of pygeoweaver
To run in CLI mode. 
"""
import logging
from unittest.mock import patch
import requests
from pygeoweaver import start, stop

import unittest

from pygeoweaver.constants import GEOWEAVER_DEFAULT_ENDPOINT_URL
from pygeoweaver.server import show
from pygeoweaver.utils import get_logger


logger = get_logger(__name__)


class TestServer(unittest.TestCase):
    def test_server_start_stop(self):
        start()
        response = requests.get(GEOWEAVER_DEFAULT_ENDPOINT_URL)
        self.assertEqual(
            response.status_code,
            200,
            f"Failed to access URL: {GEOWEAVER_DEFAULT_ENDPOINT_URL}",
        )
        stop()
        with self.assertRaises(requests.exceptions.ConnectionError):
            response = requests.get(GEOWEAVER_DEFAULT_ENDPOINT_URL)

        stop()  # stop again should have no issue

    def test_windows(self):
        with patch("pygeoweaver.server.checkOS") as mock_checkos:
            mock_checkos.return_value = 3
            with self.assertRaises(RuntimeError):
                start()
            with self.assertRaises(RuntimeError):
                stop()

    def test_show_gui(self):
        with patch("pygeoweaver.webbrowser.open") as mock_browser_open:
            show()
            mock_browser_open.assert_called_once()

            with patch("pygeoweaver.server.checkIPython") as mock_checkipython:
                mock_checkipython.return_value = True
                show()
                mock_browser_open.assert_called_once()


if __name__ == "__main__":
    unittest.main()



import subprocess
from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
)


def import_workflow(workflow_zip_file_path):
    """
    Usage: <main class> import workflow <workflow_zip_file_path>
    import a workflow from file
          <workflow_zip_file_path>
            Geoweaver workflow zip file path
    """
    if not workflow_zip_file_path:
        raise RuntimeError("Workflow zip file path is missing")
    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "import",
            "workflow",
            workflow_zip_file_path,
        ],
        cwd=f"{get_root_dir()}/",
    )

def import_workflow_from_github(git_repo_url):
    pass



import getpass
import json
import os
import subprocess

import requests

from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
)
from pygeoweaver.constants import *


def run_process(
    *,
    process_id: str,
    host_id: str,
    password: str = None,
    environment: str = None,
    sync_path: os.PathLike = None,
):
    """
    Run a process

    Args: process_id - required
        host_id - required
        password - optional
        environment - optional
    """
    if password is None:
        # prompt to ask for password
        password = getpass.getpass(f"Enter password for host - {host_id}: ")

    if sync_path:
        if not os.path.exists(sync_path):
            print("The specified path does nto exists")
        print("Updating code on workflow with the given file path.\n")
        f = open(sync_path, "r")
        context = f.read()
        f.close()
        details = requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/detail",
            data={"type": "process", "id": process_id},
        ).json()
        details["code"] = context
        requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/edit/process",
            data=json.dumps(details),
            headers={"Content-Type": "application/json"},
        )

    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "run",
            "process",
            f"--host={host_id}",
            f"--password={password}",
            f"--environment={environment}",
            process_id,
        ],
        cwd=f"{get_root_dir()}/",
    )


def run_workflow(
    *,
    workflow_id: str,
    workflow_folder_path: str = None,
    workflow_zip_file_path: str = None,
    environment_list: str = None,
    host_list: str = None,
    password_list: str = None,
    sync_path: os.PathLike = None,
):
    """
    Usage: <main class> run workflow [-d=<workflowFolderPath>]
                                    [-f=<workflowZipPath>] [-e=<envs>]...
                                    [-h=<hostStrings>]... [-p=<passes>]...
                                    <workflowId>
        <workflowId>           workflow id to run
    -d, --workflow-folder-path=<workflowFolderPath>
                                geoweaver workflow folder path
    -e, --environments=<envs>  environments to run on. List of environment ids with comma as separator
    -f, --workflow-zip-file-path=<workflowZipPath>
                                workflow package or path to workflow zip to run
    -h, --hosts=<hostStrings>  hosts to run on. list of host ids with comma as separator.
    -p, --passwords=<passes>   passwords to the target hosts. list of passwords with comma as separator.
    """
    download_geoweaver_jar()

    if password_list is None:
        # prompt to ask for password
        password_list = []
        for host in host_list.split(","):
            password = getpass.getpass(f"Enter password for host - {host}: ")
            password_list.append(password)
    elif len(password_list.split(",")) != len(host_list.split(",")):
        raise RuntimeError("The password list length doesn't match host list")

    password_list = ",".join(password_list)
    if sync_path:
        from . import sync_workflow

        sync_workflow(workflow_id=workflow_id, sync_to_path=sync_path)

    if not workflow_id and not workflow_folder_path and not workflow_zip_file_path:
        raise RuntimeError(
            "Please provide at least one of the three options: workflow id, "
            "folder path or zip path"
        )

    if workflow_id and not workflow_folder_path and not workflow_zip_file_path:
        command = [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "run",
            "workflow",
            workflow_id,
            "-h",
            host_list,
            "-p",
            password_list,
        ]
        if environment_list:
            command.extend(["-e", environment_list])
        subprocess.run(command, cwd=f"{get_root_dir()}/")

    if workflow_folder_path and not workflow_zip_file_path:
        # command to run workflow from folder
        command = [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "run",
            "workflow",
            workflow_id,
            "-d",
            workflow_folder_path,
            "-h",
            host_list,
            "-p",
            password_list
        ]
        if environment_list:
            command.extend(["-e", environment_list])
        subprocess.run(command, cwd=f"{get_root_dir()}/")

    if not workflow_folder_path and workflow_zip_file_path:
        command = [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "run",
            "workflow",
            workflow_id,
            "-f",
            workflow_zip_file_path,
            "-h",
            host_list,
            "-p",
            password_list,
        ]
        if environment_list:
            command.extend(["-e", environment_list])
        subprocess.run(command, cwd=f"{get_root_dir()}/")



import os
import json
import zipfile

import requests
import typing

from pygeoweaver.constants import *
from pygeoweaver.utils import (
    download_geoweaver_jar,
    copy_files,
)


def sync(process_id: str, local_path: typing.Union[str, os.PathLike], direction: str):
    print(f"Proceeding with {direction}\n")
    if direction == "download":
        if not local_path:
            raise Exception("Sync path not found.")
        r = requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/detail",
            data={"type": "process", "id": process_id},
        ).json()
        code = r["code"]
        decoded_string = code
        file_name = r["name"]
        ext = None
        if r["lang"] == "python":
            ext = ".py"
        elif r["lang"] == "shell":
            ext = ".sh"
        elif r["lang"] == "jupyter":
            ext = "ipynb"
        else:
            raise Exception("Unknown file format.")
        with open(os.path.join(local_path, file_name + ext), "w") as file:
            file.write(decoded_string)
        print(f"Wrote file {file_name + ext} to {local_path}")
    elif direction == "upload":
        if not local_path:
            raise Exception("Sync path not found.")
        process_prev_state = requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/detail",
            data={"type": "process", "id": process_id},
        ).json()
        with open(local_path, "r") as f:
            f_content = f.read()
            process_prev_state["code"] = f_content
            response = requests.post(
                f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/edit/process",
                data=json.dumps(process_prev_state),
                headers={"Content-Type": "application/json"},
            )
            if response.status_code == 200:
                print("Process update was successful")
            else:
                print("Update failed with status code:", response.status_code)
    else:
        raise Exception(
            "Please specify the direction to sync. Choices - [UPLOAD, DOWNLOAD]"
        )


def sync_workflow(workflow_id: str, sync_to_path: typing.Union[str, os.PathLike]):
    download_geoweaver_jar()
    # download workflow
    r = requests.post(
        f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/downloadworkflow",
        data={"id": workflow_id, "option": "workflowwithprocesscodeallhistory"},
    ).text
    filename = r.rsplit("/")[-1]
    home_dir = os.path.expanduser("~")
    tmp_dir = os.path.join(home_dir, "tmp")
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)
    # unzip the workflow
    with zipfile.ZipFile(
        os.path.join(home_dir, "gw-workspace", "temp", filename)
    ) as ref:
        ref.extractall(os.path.join(home_dir, "tmp"))
    # check if target workflow path and the unzipped workflow match
    if not sync_to_path:
        raise Exception(
            "Please provide path to workflow that you wish to sync code and history"
        )
    import_id = json.loads(
        open(os.path.join(home_dir, "tmp", "workflow.json"), "r").read()
    ).get("id")
    sync_id = json.loads(
        open(os.path.join(sync_to_path, "workflow.json"), "r").read()
    ).get("id")

    if import_id == sync_id:
        # if they match perform file replace
        copy_files(
            source_folder=os.path.join(home_dir, "tmp"), destination_folder=sync_to_path
        )
    else:
        print("Workflow ID mismatch, please check the `sync_to_path` path.")



import platform
import os
import subprocess
import shutil
import sys
import tarfile
import zipfile
import urllib.request

from pygeoweaver.utils import get_home_dir, get_java_bin_path


def install_jdk():
    system = platform.system()
    architecture = platform.machine()

    if system == "Darwin":
        if architecture == "x86_64":
            install_jdk_macos("11.0.18-10", "jdk_x64_mac_hotspot")
        elif architecture == "arm64":
            install_jdk_macos("11.0.18-10", "jdk_aarch64_mac_hotspot")
        else:
            print("Unsupported architecture.")

    elif system == "Linux":
        if architecture == "x86_64":
            install_jdk_linux("11.0.18-10", "jdk_x64_linux_hotspot")
        elif architecture == "aarch64":
            install_jdk_linux("11.0.18-10", "jdk_aarch64_linux_hotspot")
        else:
            print("Unsupported architecture.")

    elif system == "Windows":
        if architecture == "AMD64" or architecture == "x86_64":
            install_jdk_windows("11.0.18-10", "jdk_x64_windows_hotspot")
        elif architecture == "x86-32":
            install_jdk_windows("11.0.18-10", "jdk_x86-32_windows_hotspot")
        else:
            print("Unsupported architecture.")

    else:
        print("Unsupported platform.")


def install_jdk_macos(jdk_version, jdk_arch):
    # jdk_aarch64_linux_hotspot
    jdk_url = f'https://github.com/adoptium/temurin11-binaries/releases/download/jdk-{jdk_version.replace("-", "%2B")}/OpenJDK11U-{jdk_arch}_{jdk_version.replace("-", "_")}.tar.gz'
    jdk_install_dir = os.path.expanduser("~/jdk")

    # Download JDK archive
    download_file(jdk_url, f"{get_home_dir()}/jdk.tar.gz")

    # Extract JDK archive
    extract_tar_archive(f"{get_home_dir()}/jdk.tar.gz", jdk_install_dir)

    # Set JDK environment variables
    set_jdk_env_vars(f'{jdk_install_dir}/jdk-{jdk_version.replace("-", "+")}')


def install_jdk_linux(jdk_version, jdk_arch):
    jdk_url = f'https://github.com/adoptium/temurin11-binaries/releases/download/jdk-{jdk_version.replace("-", "%2B")}/OpenJDK11U-{jdk_arch}_{jdk_version.replace("-", "_")}.tar.gz'
    jdk_install_dir = os.path.expanduser("~/jdk")

    # Download JDK archive
    download_file(jdk_url, f"{get_home_dir()}/jdk.tar.gz")

    # Extract JDK archive
    extract_tar_archive(f"{get_home_dir()}/jdk.tar.gz", jdk_install_dir)

    # Set JDK environment variables
    set_jdk_env_vars(f'{jdk_install_dir}/jdk-{jdk_version.replace("-", "+")}')


def install_jdk_windows(jdk_version, jdk_arch):
    jdk_url = f'https://github.com/adoptium/temurin11-binaries/releases/download/jdk-{jdk_version.replace("-", "%2B")}/OpenJDK11U-{jdk_arch}_{jdk_version.replace("-", "_")}.zip'
    jdk_install_dir = os.path.expanduser("~/jdk")

    # Download JDK archive
    download_file(jdk_url, f"{get_home_dir()}/jdk.zip")

    # Extract JDK archive
    extract_zip_archive(f"{get_home_dir()}/jdk.zip", jdk_install_dir)

    # Set JDK environment variables
    set_jdk_env_vars(f'{jdk_install_dir}/jdk-{jdk_version.replace("-", "+")}')


def download_file(url, filename):
    if os.path.exists(filename):
        print(f"{filename} already exists.")
        return
    print(f"Downloading {filename}...", url)
    urllib.request.urlretrieve(url, filename)
    print(f"{filename} downloaded.")


def extract_tar_archive(archive_path, destination_dir):
    if not os.path.exists(destination_dir):
        print(f"Extracting {archive_path}...")
        with tarfile.open(archive_path, "r:gz") as tar_ref:
            tar_ref.extractall(destination_dir)
        print(f"{archive_path} extracted to {destination_dir}.")


def extract_zip_archive(archive_path, destination_dir):
    print(f"Extracting {archive_path}...")
    with zipfile.ZipFile(archive_path, "r") as zip_ref:
        zip_ref.extractall(destination_dir)
    print(f"{archive_path} extracted to {destination_dir}.")


def set_jdk_env_vars(jdk_install_dir):
    print(f"Setting JDK environment variables...")
    java_line = f'\nexport JAVA_HOME="{jdk_install_dir}"\n'

    check_java = False
    with open(os.path.expanduser("~/.bashrc"), "r") as file:
        for line in file:
            if line.strip() == java_line:
                check_java = True
                break

    if not check_java:
        with open(os.path.expanduser("~/.bashrc"), "a") as bashrc:
            bashrc.write(f'export JAVA_HOME="{jdk_install_dir}"\n')
            bashrc.write(f'export PATH="$JAVA_HOME/bin:$PATH"\n')
            print("JDK environment variables set.")

    subprocess.run(["bash", "-c", "source ~/.bashrc"])


def install_java():
    system = platform.system()
    if system == "Darwin":
        os.system(
            "/bin/bash -c '/usr/bin/ruby -e \"$(curl -fsSL "
            "https://raw.githubusercontent.com/Homebrew/install/master/install)\"'"
        )
        os.system("brew install openjdk")
    elif system == "Linux":
        # need to check if the package manager type is apt or yum
        # arch / debian
        package_manager = None
        if os.path.exists("/usr/bin/apt"):
            package_manager = "apt"
        elif os.path.exists("/usr/bin/yum"):
            package_manager = "yum"

        if package_manager:
            os.system(f"sudo {package_manager} update")
            os.system(f"sudo {package_manager} install -y default-jre default-jdk")
        else:
            print("Package manager not found. Unable to install Java.")
            sys.exit(1)
    elif system == "Windows":
        # note: this requires admin access to the pc, else it will fail saying
        # Access to the path 'C:\ProgramData\chocolatey\lib-bad' is denied.
        os.system(
            'powershell -Command "Set-ExecutionPolicy Bypass -Scope Process -Force; ['
            "System.Net.ServicePointManager]::SecurityProtocol = 3072; iex ((New-Object "
            "System.Net.WebClient).DownloadString('https://chocolatey.org/install.ps1'))\""
        )
        os.system("choco install -y openjdk")
    else:
        print("Unsupported operating system.")
        sys.exit(1)


def is_java_installed():
    try:
        # Check if Java is installed by running "java -version" command
        subprocess.run(
            [get_java_bin_path(), "-version"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        return True
    except:
        return False


def check_java():
    # Check if Java is installed
    if not is_java_installed():
        print("Java is not installed. Installing...")
        install_jdk()
        print("Java installation complete.")



import json

import requests
import subprocess
from pygeoweaver.constants import *
from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
    check_ipython,
)
import pandas as pd


def list_hosts():
    download_geoweaver_jar()
    subprocess.run(
        [get_java_bin_path(), "-jar", get_geoweaver_jar_path(), "list", "--host"],
        cwd=f"{get_root_dir()}/",
    )


def list_processes():
    download_geoweaver_jar()
    subprocess.run(["chmod", "+x", get_geoweaver_jar_path()], cwd=f"{get_root_dir()}/")
    subprocess.run(
        [get_java_bin_path(), "-jar", get_geoweaver_jar_path(), "list", "--process"],
        cwd=f"{get_root_dir()}/",
    )


def list_processes_in_workflow(workflow_id):
    download_geoweaver_jar()
    payload = {"id": workflow_id, "type": "workflow"}
    r = requests.post(
        f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/detail", data=payload
    )
    nodes = json.loads(r.json()["nodes"])
    result = [
        {"title": item["title"], "id": item["id"].split(".")[0]} for item in nodes
    ]

    if check_ipython():
        return pd.DataFrame(result)
    return result


def list_workflows():
    download_geoweaver_jar()
    subprocess.run(
        [get_java_bin_path(), "-jar", get_geoweaver_jar_path(), "list", "--workflow"],
        cwd=f"{get_root_dir()}/",
    )



from io import StringIO
import sys
import unittest
from pygeoweaver.sc_detail import detail_host, detail_process, detail_workflow
from pygeoweaver.utils import get_logger


logger = get_logger(__name__)


def test_detail_process(capfd):
    detail_process("not_existing_id")
    output, err = capfd.readouterr()
    logger.debug("stdout_output" + output)
    assert "No process found with id: not_existing_id" in output


def test_detail_workflow(capfd):
    detail_workflow("not_existing_id")
    output, err = capfd.readouterr()
    logger.debug("stdout_output" + output)
    assert "No workflow found with id: not_existing_id" in output


def test_detail_host(capfd):
    detail_host("not_existing_id")
    output, err = capfd.readouterr()
    logger.debug("stdout_output" + output)
    assert "No host found with id: not_existing_id" in output



import subprocess

import pandas as pd
import requests

from pygeoweaver.constants import *
from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
)


def show_history(history_id):
    """
    Workflow and process history uses the same method to check
    """
    if not history_id:
        raise RuntimeError("history id is missing")
    download_geoweaver_jar()
    subprocess.run(
        [get_java_bin_path(), "-jar", get_geoweaver_jar_path(), "history", history_id],
        cwd=f"{get_root_dir()}/",
    )


def get_process_history(process_id):
    """
        Get list of history for a process using process id
    :param process_id: str    :type process_id: str
    """
    if not process_id:
        raise Exception("please pass `process_id` as a parameter to the function.")
    download_geoweaver_jar()
    try:
        r = requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/logs",
            data={"type": "process", "id": process_id},
        ).json()
        df = pd.DataFrame(r)
        df["history_begin_time"] = pd.to_datetime(df["history_begin_time"], unit="ms")
        df["history_end_time"] = pd.to_datetime(df["history_end_time"], unit="ms")
        return df
    except Exception as e:
        subprocess.run(
            f"{get_java_bin_path()} -jar {get_geoweaver_jar_path()} process-history {process_id}",
            cwd=f"{get_root_dir()}/",
            shell=True,
        )


def get_workflow_history(workflow_id):
    """
        Get list of history for a workflow using workflow id
    :param workflow_id: str
    """
    if not workflow_id:
        raise Exception("please pass `workflow_id` as a parameter to the function.")
    try:
        r = requests.post(
            f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/logs",
            data={"type": "workflow", "id": workflow_id},
        ).json()
        df = pd.DataFrame(r)
        df["history_begin_time"] = pd.to_datetime(df["history_begin_time"], unit="ms")
        df["history_end_time"] = pd.to_datetime(df["history_end_time"], unit="ms")
        return df
    except Exception as e:
        subprocess.run(
            f"{get_java_bin_path()} -jar {get_geoweaver_jar_path()} workflow-history {workflow_id}",
            shell=True,
            cwd=f"{get_root_dir()}/",
        )



"""
Detail subcommand
"""

import subprocess

import requests
from pygeoweaver.constants import *

from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
)


def detail_workflow(workflow_id):
    if not workflow_id:
        raise RuntimeError("Workflow id is missing")
    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "detail",
            f"--workflow-id={workflow_id}",
        ],
        cwd=f"{get_root_dir()}/",
    )


def detail_process(process_id):
    if not process_id:
        raise RuntimeError("Process id is missing")
    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "detail",
            f"--process-id={process_id}",
        ],
        cwd=f"{get_root_dir()}/",
    )


def detail_host(host_id):
    if not host_id:
        raise RuntimeError("Host id is missing")
    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "detail",
            f"--host-id={host_id}",
        ],
        cwd=f"{get_root_dir()}/",
    )


def get_process_code(process_id):
    r = requests.post(
        f"{GEOWEAVER_DEFAULT_ENDPOINT_URL}/web/detail",
        data={"type": "process", "id": process_id},
    ).json()
    return r["code"]



import os.path
import zipfile
import subprocess
from pygeoweaver.utils import (
    download_geoweaver_jar,
    get_geoweaver_jar_path,
    get_java_bin_path,
    get_root_dir,
)


def export_workflow(
    workflow_id, mode=4, target_file_path=None, unzip=False, unzip_directory_name=None
):
    """
    Usage: <main class> export workflow [--mode=<export_mode>] <workflow_id>
                                    <target_file_path>
      <workflow_id>          Geoweaver workflow ID
      <target_file_path>     target file path to save the workflow zip
      --mode=<export_mode>   exportation model options:
                                1 - workflow only
                                2 - workflow with process code
                                3 - workflow with process code and only good
                               history
                                4 - workflow with process code and all the
                               history.default option is 4.
    """
    if not workflow_id:
        raise RuntimeError("Workflow id is missing")
    download_geoweaver_jar()
    subprocess.run(
        [
            get_java_bin_path(),
            "-jar",
            get_geoweaver_jar_path(),
            "export",
            "workflow",
            f"--mode={mode}",
            workflow_id,
            target_file_path,
        ],
        cwd=f"{get_root_dir()}/",
    )
    if unzip:
        if not unzip_directory_name:
            raise Exception("Please provide unzip directory name")
        with zipfile.ZipFile(target_file_path, "r") as zip_ref:
            zip_ref.extractall(
                os.path.join(os.path.dirname(target_file_path), unzip_directory_name)
            )



import os
import sys
import shutil
import logging
import subprocess
import requests
import platform

from IPython import get_ipython


def get_home_dir():
    return os.path.expanduser("~")


def get_root_dir():
    head, tail = os.path.split(__file__)
    return head


def get_java_bin_from_which():
    system = platform.system()

    if system == "Darwin" or system == "Linux":

        try:

            java_bin_sh = f"{get_root_dir()}/java_bin.sh"

            os.chmod(java_bin_sh, 0o755)

            output = subprocess.check_output([java_bin_sh], encoding="utf-8")

            java_bin_path = output.strip()

        except subprocess.CalledProcessError as e:

            print(f"Command execution failed: {e.output}")

            return None

    elif system == "Windows":

        print("Unsupported platform for windows yet.")

    else:
        print("Unsupported platform.")

    return java_bin_path


def get_java_bin_path():
    # Check if the 'java' command is available in the system path
    if sys.platform.startswith("win"):  # Windows
        java_exe = "java.exe"
    else:
        java_exe = "java"

    java_bin_path = None

    for path in os.environ.get("PATH", "").split(os.pathsep):
        bin_path = os.path.join(path, java_exe)
        if os.path.isfile(bin_path) and os.access(bin_path, os.X_OK):
            java_bin_path = bin_path
            break

    if java_bin_path is None:
        java_bin_path = get_java_bin_from_which()

    return java_bin_path


def get_module_absolute_path():
    module_path = os.path.abspath(__file__)
    return os.path.dirname(module_path)


def get_geoweaver_jar_path():
    return f"{get_home_dir()}/geoweaver.jar"


def check_geoweaver_jar():
    return os.path.isfile(get_geoweaver_jar_path())


def download_geoweaver_jar(overwrite=False):
    if check_geoweaver_jar():
        if overwrite:
            os.remove(get_geoweaver_jar_path())
        else:
            system = platform.system()
            if not system == "Windows":  # Windows files are exec by default
                subprocess.run(
                    ["chmod", "+x", get_geoweaver_jar_path()], cwd=f"{get_root_dir()}/"
                )
                return

    print("Downloading latest version of Geoweaver...")
    geoweaver_url = (
        "https://github.com/ESIPFed/Geoweaver/releases/download/latest/geoweaver.jar"
    )
    r = requests.get(geoweaver_url)

    with open(get_geoweaver_jar_path(), "wb") as f:
        f.write(r.content)

    if check_geoweaver_jar():
        print("Geoweaver.jar is downloaded")

    else:
        raise RuntimeError("Fail to download geoweaver.jar")


def check_os():
    if platform.system() == "Linux" or platform == "Linux2":
        return 1
    elif platform.system() == "Darwin":
        return 2
    elif platform.system() == "Windows":
        return 3


def check_ipython():
    try:
        return get_ipython().__class__.__name__ == "ZMQInteractiveShell"
    except:
        return False


def get_logger(class_name):
    logger = logging.getLogger(class_name)
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)
    return logger


def copy_files(source_folder, destination_folder):
    for root, dirs, files in os.walk(source_folder):
        for file in files:
            source_file = os.path.join(root, file)
            destination_file = os.path.join(
                destination_folder, os.path.relpath(source_file, source_folder)
            )
            os.makedirs(os.path.dirname(destination_file), exist_ok=True)
            shutil.copy2(source_file, destination_file)



import os
import socket
import subprocess
import webbrowser
from pygeoweaver.constants import GEOWEAVER_DEFAULT_ENDPOINT_URL
from pygeoweaver.jdk_utils import check_java
from pygeoweaver.utils import (
    check_ipython,
    check_os,
    download_geoweaver_jar,
    get_logger,
    get_module_absolute_path,
    get_root_dir,
)

"""
This module provides function to start and stop Geoweaver server.
If it detects the current environment is Jupyter notebook, it will 
open Geoweaver GUI in the output cell (if gui is not disabld.)

"""

logger = get_logger(__name__)


def start(force=False):
    download_geoweaver_jar(overwrite=force)
    check_java()

    if check_os() == 3:
        subprocess.run(
            [f"{get_module_absolute_path()}/start.bat"], cwd=f"{get_root_dir()}/"
        )
    else:
        subprocess.run(
            [f"{get_module_absolute_path()}/start.sh"], cwd=f"{get_root_dir()}/"
        )


def stop():
    check_java()
    if check_os() == 3:
        subprocess.run(
            [f"{get_module_absolute_path()}/stop.bat"],
            cwd=f"{get_root_dir()}/",
            shell=True,
        )
    else:
        subprocess.run(
            [f"{get_module_absolute_path()}/stop.sh"],
            cwd=f"{get_root_dir()}/",
            shell=True,
        )


def show(geoweaver_url=GEOWEAVER_DEFAULT_ENDPOINT_URL):
    download_geoweaver_jar()  # check if geoweaver is initialized
    check_java()
    if check_ipython():
        logger.info("enter ipython block")
        from IPython.display import IFrame

        logger.warning("This only works when the Jupyter is visited from localhost!")
        return IFrame(src=geoweaver_url, width="100%", height="500px")
    else:
        logger.info("enter self opening block")
        webbrowser.open(geoweaver_url)



"""
All functions of subcommand interfaces
"""
from pygeoweaver.sc_detail import *
from pygeoweaver.sc_list import *
from pygeoweaver.sc_export import *
from pygeoweaver.sc_history import *
from pygeoweaver.sc_import import *
from pygeoweaver.sc_list import *
from pygeoweaver.sc_run import *
from pygeoweaver.server import *
from pygeoweaver.sc_resetpassword import *
from pygeoweaver.sc_help import *
from pygeoweaver.sc_create import *
from pygeoweaver.sc_sync import *
from pygeoweaver.sc_find import *
